@@ -51,7 +51,7 @@ pub fn render_tab(
     let right_separator = style!(background_color, foreground_color).paint(separator);
     let tab_styled_text = if !focused_clients.is_empty() {
         let (cursor_section, extra_length) = cursors(focused_clients, palette);
-        tab_text_len += extra_length;
+        tab_text_len += extra_length + 2; // 2 for cursor_beginning and cursor_end
         let mut s = String::new();
         let cursor_beginning = style!(foreground_color, background_color)
             .bold()
@@ -89,8 +89,10 @@ pub fn tab_style(
 ) -> LinePart {
     let separator = tab_separator(capabilities);
 
-    if tab.is_sync_panes_active {
-        tabname.push_str(" (Sync)");
+    if tab.is_fullscreen_active {
+        tabname.push_str(" (FULLSCREEN)");
+    } else if tab.is_sync_panes_active {
+        tabname.push_str(" (SYNC)");
     }
     // we only color alternate tabs differently if we can't use the arrow fonts to separate them
     if !capabilities.arrow_fonts {
