@@ -6,7 +6,7 @@ pub mod sixel;
 pub mod terminal_character;
 
 mod active_panes;
-mod floating_panes;
+pub mod floating_panes;
 mod plugin_pane;
 mod search;
 mod terminal_pane;
