@@ -10,10 +10,9 @@ use std::{
     collections::{BTreeMap, HashMap, HashSet},
     fs,
     path::PathBuf,
-    sync::{Arc, Mutex},
     time::Duration,
 };
-use wasmer::Store;
+use wasmtime::Engine;
 
 use crate::panes::PaneId;
 use crate::screen::ScreenInstruction;
@@ -26,12 +25,13 @@ use wasm_bridge::WasmBridge;
 use zellij_utils::{
     async_std::{channel, future::timeout, task},
     data::{
-        Event, EventType, MessageToPlugin, PermissionStatus, PermissionType, PipeMessage,
-        PipeSource, PluginCapabilities,
+        ClientInfo, Event, EventType, InputMode, MessageToPlugin, PermissionStatus, PermissionType,
+        PipeMessage, PipeSource, PluginCapabilities,
     },
     errors::{prelude::*, ContextType, PluginContext},
     input::{
         command::TerminalAction,
+        keybinds::Keybinds,
         layout::{FloatingPaneLayout, Layout, Run, RunPlugin, RunPluginOrAlias, TiledPaneLayout},
         plugins::PluginAliases,
     },
@@ -49,13 +49,14 @@ pub enum PluginInstruction {
         bool,           // should be opened in place
         Option<String>, // pane title
         RunPluginOrAlias,
-        usize,          // tab index
+        Option<usize>,  // tab index
         Option<PaneId>, // pane id to replace if this is to be opened "in-place"
         ClientId,
         Size,
         Option<PathBuf>, // cwd
         bool,            // skip cache
     ),
+    LoadBackgroundPlugin(RunPluginOrAlias, ClientId),
     Update(Vec<(Option<PluginId>, Option<ClientId>, Event)>), // Focused plugin / broadcast, client_id, event data
     Unload(PluginId),                                         // plugin_id
     Reload(
@@ -65,6 +66,7 @@ pub enum PluginInstruction {
         usize, // tab index
         Size,
     ),
+    ReloadPluginWithId(u32),
     Resize(PluginId, usize, usize), // plugin_id, columns, rows
     AddClient(ClientId),
     RemoveClient(ClientId),
@@ -74,6 +76,7 @@ pub enum PluginInstruction {
         Option<TiledPaneLayout>,
         Vec<FloatingPaneLayout>,
         usize, // tab_index
+        bool,  // should change focus to new tab
         ClientId,
     ),
     ApplyCachedEvents {
@@ -134,6 +137,7 @@ pub enum PluginInstruction {
         cwd: Option<PathBuf>,
         skip_cache: bool,
         cli_client_id: ClientId,
+        plugin_and_client_id: Option<(u32, ClientId)>,
     },
     CachePluginEvents {
         plugin_id: PluginId,
@@ -143,7 +147,17 @@ pub enum PluginInstruction {
         message: MessageToPlugin,
     },
     UnblockCliPipes(Vec<PluginRenderAsset>),
+    Reconfigure {
+        client_id: ClientId,
+        keybinds: Option<Keybinds>,
+        default_mode: Option<InputMode>,
+        default_shell: Option<TerminalAction>,
+    },
+    FailedToWriteConfigToDisk {
+        file_path: Option<PathBuf>,
+    },
     WatchFilesystem,
+    ListClientsToPlugin(SessionLayoutMetadata, PluginId, ClientId),
     Exit,
 }
 
@@ -151,9 +165,11 @@ impl From<&PluginInstruction> for PluginContext {
     fn from(plugin_instruction: &PluginInstruction) -> Self {
         match *plugin_instruction {
             PluginInstruction::Load(..) => PluginContext::Load,
+            PluginInstruction::LoadBackgroundPlugin(..) => PluginContext::LoadBackgroundPlugin,
             PluginInstruction::Update(..) => PluginContext::Update,
             PluginInstruction::Unload(..) => PluginContext::Unload,
             PluginInstruction::Reload(..) => PluginContext::Reload,
+            PluginInstruction::ReloadPluginWithId(..) => PluginContext::ReloadPluginWithId,
             PluginInstruction::Resize(..) => PluginContext::Resize,
             PluginInstruction::Exit => PluginContext::Exit,
             PluginInstruction::AddClient(_) => PluginContext::AddClient,
@@ -183,13 +199,18 @@ impl From<&PluginInstruction> for PluginContext {
             PluginInstruction::WatchFilesystem => PluginContext::WatchFilesystem,
             PluginInstruction::KeybindPipe { .. } => PluginContext::KeybindPipe,
             PluginInstruction::DumpLayoutToPlugin(..) => PluginContext::DumpLayoutToPlugin,
+            PluginInstruction::Reconfigure { .. } => PluginContext::Reconfigure,
+            PluginInstruction::FailedToWriteConfigToDisk { .. } => {
+                PluginContext::FailedToWriteConfigToDisk
+            },
+            PluginInstruction::ListClientsToPlugin(..) => PluginContext::ListClientsToPlugin,
         }
     }
 }
 
 pub(crate) fn plugin_thread_main(
     bus: Bus<PluginInstruction>,
-    store: Store,
+    engine: Engine,
     data_dir: PathBuf,
     mut layout: Box<Layout>,
     layout_dir: Option<PathBuf>,
@@ -199,12 +220,19 @@ pub(crate) fn plugin_thread_main(
     client_attributes: ClientAttributes,
     default_shell: Option<TerminalAction>,
     plugin_aliases: Box<PluginAliases>,
+    default_mode: InputMode,
+    default_keybinds: Keybinds,
+    background_plugins: HashSet<RunPluginOrAlias>,
+    // the client id that started the session,
+    // we need it here because the thread's own list of connected clients might not yet be updated
+    // on session start when we need to load the background plugins, and so we must have an
+    // explicit client_id that has started the session
+    initiating_client_id: ClientId,
 ) -> Result<()> {
     info!("Wasm main thread starts");
     let plugin_dir = data_dir.join("plugins/");
     let plugin_global_data_dir = plugin_dir.join("data");
     layout.populate_plugin_aliases_in_layout(&plugin_aliases);
-    let store = Arc::new(Mutex::new(store));
 
     // use this channel to ensure that tasks spawned from this thread terminate before exiting
     // https://tokio.rs/tokio/topics/shutdown#waiting-for-things-to-finish-shutting-down
@@ -212,7 +240,7 @@ pub(crate) fn plugin_thread_main(
 
     let mut wasm_bridge = WasmBridge::new(
         bus.senders.clone(),
-        store,
+        engine,
         plugin_dir,
         path_to_default_shell,
         zellij_cwd,
@@ -221,8 +249,20 @@ pub(crate) fn plugin_thread_main(
         default_shell,
         layout.clone(),
         layout_dir,
+        default_mode,
+        default_keybinds,
     );
 
+    for run_plugin_or_alias in background_plugins {
+        load_background_plugin(
+            run_plugin_or_alias,
+            &mut wasm_bridge,
+            &bus,
+            &plugin_aliases,
+            initiating_client_id,
+        );
+    }
+
     loop {
         let (event, mut err_ctx) = bus.recv().expect("failed to receive event on channel");
         err_ctx.add_call(ContextType::Plugin((&event).into()));
@@ -242,9 +282,10 @@ pub(crate) fn plugin_thread_main(
                 run_plugin_or_alias.populate_run_plugin_if_needed(&plugin_aliases);
                 let cwd = run_plugin_or_alias.get_initial_cwd().or(cwd);
                 let run_plugin = run_plugin_or_alias.get_run_plugin();
+                let start_suppressed = false;
                 match wasm_bridge.load_plugin(
                     &run_plugin,
-                    Some(tab_index),
+                    tab_index,
                     size,
                     cwd.clone(),
                     skip_cache,
@@ -257,10 +298,11 @@ pub(crate) fn plugin_thread_main(
                             should_be_open_in_place,
                             run_plugin_or_alias,
                             pane_title,
-                            Some(tab_index),
+                            tab_index,
                             plugin_id,
                             pane_id_to_replace,
                             cwd,
+                            start_suppressed,
                             Some(client_id),
                         )));
                     },
@@ -269,6 +311,15 @@ pub(crate) fn plugin_thread_main(
                     },
                 }
             },
+            PluginInstruction::LoadBackgroundPlugin(run_plugin_or_alias, client_id) => {
+                load_background_plugin(
+                    run_plugin_or_alias,
+                    &mut wasm_bridge,
+                    &bus,
+                    &plugin_aliases,
+                    client_id,
+                );
+            },
             PluginInstruction::Update(updates) => {
                 wasm_bridge.update_plugins(updates, shutdown_send.clone())?;
             },
@@ -300,6 +351,7 @@ pub(crate) fn plugin_thread_main(
                                     // we intentionally do not provide the client_id here because it belongs to
                                     // the cli who spawned the command and is not an existing client_id
                                     let skip_cache = true; // when reloading we always skip cache
+                                    let start_suppressed = false;
                                     match wasm_bridge.load_plugin(
                                         &Some(run_plugin),
                                         Some(tab_index),
@@ -321,6 +373,7 @@ pub(crate) fn plugin_thread_main(
                                                     plugin_id,
                                                     None,
                                                     None,
+                                                    start_suppressed,
                                                     None,
                                                 ),
                                             ));
@@ -341,6 +394,9 @@ pub(crate) fn plugin_thread_main(
                     },
                 }
             },
+            PluginInstruction::ReloadPluginWithId(plugin_id) => {
+                wasm_bridge.reload_plugin_with_id(plugin_id).non_fatal();
+            },
             PluginInstruction::Resize(pid, new_columns, new_rows) => {
                 wasm_bridge.resize_plugin(pid, new_columns, new_rows, shutdown_send.clone())?;
             },
@@ -356,13 +412,28 @@ pub(crate) fn plugin_thread_main(
                 mut tab_layout,
                 mut floating_panes_layout,
                 tab_index,
+                should_change_focus_to_new_tab,
                 client_id,
             ) => {
+                // prefer connected clients so as to avoid opening plugins in the background for
+                // CLI clients unless no-one else is connected
+                let client_id = if wasm_bridge.client_is_connected(&client_id) {
+                    client_id
+                } else if let Some(first_client_id) = wasm_bridge.get_first_client_id() {
+                    first_client_id
+                } else {
+                    client_id
+                };
+
                 let mut plugin_ids: HashMap<RunPluginOrAlias, Vec<PluginId>> = HashMap::new();
                 tab_layout = tab_layout.or_else(|| Some(layout.new_tab().0));
-                tab_layout
-                    .as_mut()
-                    .map(|t| t.populate_plugin_aliases_in_layout(&plugin_aliases));
+                tab_layout.as_mut().map(|t| {
+                    t.populate_plugin_aliases_in_layout(&plugin_aliases);
+                    if let Some(cwd) = cwd.as_ref() {
+                        t.add_cwd_to_layout(cwd);
+                    }
+                    t
+                });
                 floating_panes_layout.iter_mut().for_each(|f| {
                     f.run
                         .as_mut()
@@ -413,6 +484,7 @@ pub(crate) fn plugin_thread_main(
                     floating_panes_layout,
                     tab_index,
                     plugin_ids,
+                    should_change_focus_to_new_tab,
                     client_id,
                 )));
             },
@@ -485,21 +557,33 @@ pub(crate) fn plugin_thread_main(
                 )?;
             },
             PluginInstruction::DumpLayout(mut session_layout_metadata, client_id) => {
-                populate_session_layout_metadata(&mut session_layout_metadata, &wasm_bridge);
+                populate_session_layout_metadata(
+                    &mut session_layout_metadata,
+                    &wasm_bridge,
+                    &plugin_aliases,
+                );
                 drop(bus.senders.send_to_pty(PtyInstruction::DumpLayout(
                     session_layout_metadata,
                     client_id,
                 )));
             },
             PluginInstruction::ListClientsMetadata(mut session_layout_metadata, client_id) => {
-                populate_session_layout_metadata(&mut session_layout_metadata, &wasm_bridge);
+                populate_session_layout_metadata(
+                    &mut session_layout_metadata,
+                    &wasm_bridge,
+                    &plugin_aliases,
+                );
                 drop(bus.senders.send_to_pty(PtyInstruction::ListClientsMetadata(
                     session_layout_metadata,
                     client_id,
                 )));
             },
             PluginInstruction::DumpLayoutToPlugin(mut session_layout_metadata, plugin_id) => {
-                populate_session_layout_metadata(&mut session_layout_metadata, &wasm_bridge);
+                populate_session_layout_metadata(
+                    &mut session_layout_metadata,
+                    &wasm_bridge,
+                    &plugin_aliases,
+                );
                 match session_serialization::serialize_session_layout(
                     session_layout_metadata.into(),
                 ) {
@@ -524,8 +608,41 @@ pub(crate) fn plugin_thread_main(
                     },
                 }
             },
+            PluginInstruction::ListClientsToPlugin(
+                mut session_layout_metadata,
+                plugin_id,
+                client_id,
+            ) => {
+                populate_session_layout_metadata(
+                    &mut session_layout_metadata,
+                    &wasm_bridge,
+                    &plugin_aliases,
+                );
+                let mut clients_metadata = session_layout_metadata.all_clients_metadata();
+                let mut client_list_for_plugin = vec![];
+                let default_editor = session_layout_metadata.default_editor.clone();
+                for (client_metadata_id, client_metadata) in clients_metadata.iter_mut() {
+                    let is_current_client = client_metadata_id == &client_id;
+                    client_list_for_plugin.push(ClientInfo::new(
+                        *client_metadata_id,
+                        client_metadata.get_pane_id().into(),
+                        client_metadata.stringify_command(&default_editor),
+                        is_current_client,
+                    ));
+                }
+                let updates = vec![(
+                    Some(plugin_id),
+                    Some(client_id),
+                    Event::ListClients(client_list_for_plugin),
+                )];
+                wasm_bridge.update_plugins(updates, shutdown_send.clone())?;
+            },
             PluginInstruction::LogLayoutToHd(mut session_layout_metadata) => {
-                populate_session_layout_metadata(&mut session_layout_metadata, &wasm_bridge);
+                populate_session_layout_metadata(
+                    &mut session_layout_metadata,
+                    &wasm_bridge,
+                    &plugin_aliases,
+                );
                 drop(
                     bus.senders
                         .send_to_pty(PtyInstruction::LogLayoutToHd(session_layout_metadata)),
@@ -595,42 +712,52 @@ pub(crate) fn plugin_thread_main(
                 cwd,
                 skip_cache,
                 cli_client_id,
+                plugin_and_client_id,
             } => {
                 let should_float = floating.unwrap_or(true);
                 let mut pipe_messages = vec![];
-                match plugin {
-                    Some(plugin_url) => {
-                        // send to specific plugin(s)
-                        pipe_to_specific_plugins(
-                            PipeSource::Keybind,
-                            &plugin_url,
-                            &configuration,
-                            &cwd,
-                            skip_cache,
-                            should_float,
-                            &pane_id_to_replace,
-                            &pane_title,
-                            Some(cli_client_id),
-                            &mut pipe_messages,
-                            &name,
-                            &payload,
-                            &args,
-                            &bus,
-                            &mut wasm_bridge,
-                            &plugin_aliases,
-                        );
-                    },
-                    None => {
-                        // no specific destination, send to all plugins
-                        pipe_to_all_plugins(
-                            PipeSource::Keybind,
-                            &name,
-                            &payload,
-                            &args,
-                            &mut wasm_bridge,
-                            &mut pipe_messages,
-                        );
-                    },
+                if let Some((plugin_id, client_id)) = plugin_and_client_id {
+                    let is_private = true;
+                    pipe_messages.push((
+                        Some(plugin_id),
+                        Some(client_id),
+                        PipeMessage::new(PipeSource::Keybind, name, &payload, &args, is_private),
+                    ));
+                } else {
+                    match plugin {
+                        Some(plugin_url) => {
+                            // send to specific plugin(s)
+                            pipe_to_specific_plugins(
+                                PipeSource::Keybind,
+                                &plugin_url,
+                                &configuration,
+                                &cwd,
+                                skip_cache,
+                                should_float,
+                                &pane_id_to_replace,
+                                &pane_title,
+                                Some(cli_client_id),
+                                &mut pipe_messages,
+                                &name,
+                                &payload,
+                                &args,
+                                &bus,
+                                &mut wasm_bridge,
+                                &plugin_aliases,
+                            );
+                        },
+                        None => {
+                            // no specific destination, send to all plugins
+                            pipe_to_all_plugins(
+                                PipeSource::Keybind,
+                                &name,
+                                &payload,
+                                &args,
+                                &mut wasm_bridge,
+                                &mut pipe_messages,
+                            );
+                        },
+                    }
                 }
                 wasm_bridge.pipe_messages(pipe_messages, shutdown_send.clone())?;
             },
@@ -734,6 +861,28 @@ pub(crate) fn plugin_thread_main(
                         .context("failed to unblock input pipe");
                 }
             },
+            PluginInstruction::Reconfigure {
+                client_id,
+                keybinds,
+                default_mode,
+                default_shell,
+            } => {
+                // TODO: notify plugins that this happened so that they can eg. rebind temporary keys that
+                // were lost
+                wasm_bridge
+                    .reconfigure(client_id, keybinds, default_mode, default_shell)
+                    .non_fatal();
+            },
+            PluginInstruction::FailedToWriteConfigToDisk { file_path } => {
+                let updates = vec![(
+                    None,
+                    None,
+                    Event::FailedToWriteConfigToDisk(file_path.map(|f| f.display().to_string())),
+                )];
+                wasm_bridge
+                    .update_plugins(updates, shutdown_send.clone())
+                    .non_fatal();
+            },
             PluginInstruction::WatchFilesystem => {
                 wasm_bridge.start_fs_watcher_if_not_started();
             },
@@ -772,6 +921,7 @@ pub(crate) fn plugin_thread_main(
 fn populate_session_layout_metadata(
     session_layout_metadata: &mut SessionLayoutMetadata,
     wasm_bridge: &WasmBridge,
+    plugin_aliases: &PluginAliases,
 ) {
     let plugin_ids = session_layout_metadata.all_plugin_ids();
     let mut plugin_ids_to_cmds: HashMap<u32, RunPlugin> = HashMap::new();
@@ -785,6 +935,7 @@ fn populate_session_layout_metadata(
         }
     }
     session_layout_metadata.update_plugin_cmds(plugin_ids_to_cmds);
+    session_layout_metadata.update_plugin_aliases_in_default_layout(plugin_aliases);
 }
 
 fn pipe_to_all_plugins(
@@ -867,6 +1018,53 @@ fn pipe_to_specific_plugins(
     }
 }
 
+fn load_background_plugin(
+    mut run_plugin_or_alias: RunPluginOrAlias,
+    wasm_bridge: &mut WasmBridge,
+    bus: &Bus<PluginInstruction>,
+    plugin_aliases: &PluginAliases,
+    client_id: ClientId,
+) {
+    run_plugin_or_alias.populate_run_plugin_if_needed(&plugin_aliases);
+    let cwd = run_plugin_or_alias.get_initial_cwd();
+    let run_plugin = run_plugin_or_alias.get_run_plugin();
+    let size = Size::default();
+    let skip_cache = false;
+    match wasm_bridge.load_plugin(
+        &run_plugin,
+        None,
+        size,
+        cwd.clone(),
+        skip_cache,
+        Some(client_id),
+        None,
+    ) {
+        Ok((plugin_id, client_id)) => {
+            let should_float = None;
+            let should_be_open_in_place = false;
+            let pane_title = None;
+            let pane_id_to_replace = None;
+            let start_suppressed = true;
+            drop(bus.senders.send_to_screen(ScreenInstruction::AddPlugin(
+                should_float,
+                should_be_open_in_place,
+                run_plugin_or_alias,
+                pane_title,
+                None,
+                plugin_id,
+                pane_id_to_replace,
+                cwd,
+                start_suppressed,
+                // None,
+                Some(client_id),
+            )));
+        },
+        Err(e) => {
+            log::error!("Failed to load plugin: {e}");
+        },
+    }
+}
+
 const EXIT_TIMEOUT: Duration = Duration::from_secs(3);
 
 #[path = "./unit/plugin_tests.rs"]
