@@ -10,17 +10,18 @@ use crate::plugins::zellij_exports::{wasi_read_string, wasi_write_object};
 use highway::{HighwayHash, PortableHash};
 use log::info;
 use std::{
-    collections::{HashMap, HashSet},
+    collections::{BTreeMap, HashMap, HashSet},
     path::PathBuf,
     str::FromStr,
     sync::{Arc, Mutex},
 };
-use wasmer::{Module, Store, Value};
+use wasmtime::{Engine, Module};
 use zellij_utils::async_channel::Sender;
 use zellij_utils::async_std::task::{self, JoinHandle};
 use zellij_utils::consts::ZELLIJ_CACHE_DIR;
-use zellij_utils::data::{PermissionStatus, PermissionType, PipeMessage, PipeSource};
+use zellij_utils::data::{InputMode, PermissionStatus, PermissionType, PipeMessage, PipeSource};
 use zellij_utils::downloader::Downloader;
+use zellij_utils::input::keybinds::Keybinds;
 use zellij_utils::input::permission::PermissionCache;
 use zellij_utils::notify_debouncer_full::{notify::RecommendedWatcher, Debouncer, FileIdMap};
 use zellij_utils::plugin_api::event::ProtobufEvent;
@@ -77,7 +78,7 @@ impl PluginRenderAsset {
 pub struct WasmBridge {
     connected_clients: Arc<Mutex<Vec<ClientId>>>,
     senders: ThreadSenders,
-    store: Arc<Mutex<Store>>,
+    engine: Engine,
     plugin_dir: PathBuf,
     plugin_cache: Arc<Mutex<HashMap<PathBuf, Module>>>,
     plugin_map: Arc<Mutex<PluginMap>>,
@@ -102,12 +103,17 @@ pub struct WasmBridge {
         HashMap<RunPluginLocation, HashMap<PluginUserConfiguration, Vec<(PluginId, ClientId)>>>,
     pending_pipes: PendingPipes,
     layout_dir: Option<PathBuf>,
+    default_mode: InputMode,
+    default_keybinds: Keybinds,
+    keybinds: HashMap<ClientId, Keybinds>,
+    base_modes: HashMap<ClientId, InputMode>,
+    downloader: Downloader,
 }
 
 impl WasmBridge {
     pub fn new(
         senders: ThreadSenders,
-        store: Arc<Mutex<Store>>,
+        engine: Engine,
         plugin_dir: PathBuf,
         path_to_default_shell: PathBuf,
         zellij_cwd: PathBuf,
@@ -116,16 +122,19 @@ impl WasmBridge {
         default_shell: Option<TerminalAction>,
         default_layout: Box<Layout>,
         layout_dir: Option<PathBuf>,
+        default_mode: InputMode,
+        default_keybinds: Keybinds,
     ) -> Self {
         let plugin_map = Arc::new(Mutex::new(PluginMap::default()));
         let connected_clients: Arc<Mutex<Vec<ClientId>>> = Arc::new(Mutex::new(vec![]));
         let plugin_cache: Arc<Mutex<HashMap<PathBuf, Module>>> =
             Arc::new(Mutex::new(HashMap::new()));
         let watcher = None;
+        let downloader = Downloader::new(ZELLIJ_CACHE_DIR.to_path_buf());
         WasmBridge {
             connected_clients,
             senders,
-            store,
+            engine,
             plugin_dir,
             plugin_cache,
             plugin_map,
@@ -146,6 +155,11 @@ impl WasmBridge {
             cached_plugin_map: HashMap::new(),
             pending_pipes: Default::default(),
             layout_dir,
+            default_mode,
+            default_keybinds,
+            keybinds: HashMap::new(),
+            base_modes: HashMap::new(),
+            downloader,
         }
     }
     pub fn load_plugin(
@@ -192,7 +206,7 @@ impl WasmBridge {
                     let plugin_dir = self.plugin_dir.clone();
                     let plugin_cache = self.plugin_cache.clone();
                     let senders = self.senders.clone();
-                    let store = self.store.clone();
+                    let engine = self.engine.clone();
                     let plugin_map = self.plugin_map.clone();
                     let connected_clients = self.connected_clients.clone();
                     let path_to_default_shell = self.path_to_default_shell.clone();
@@ -202,6 +216,17 @@ impl WasmBridge {
                     let default_shell = self.default_shell.clone();
                     let default_layout = self.default_layout.clone();
                     let layout_dir = self.layout_dir.clone();
+                    let downloader = self.downloader.clone();
+                    let default_mode = self
+                        .base_modes
+                        .get(&client_id)
+                        .copied()
+                        .unwrap_or(self.default_mode);
+                    let keybinds = self
+                        .keybinds
+                        .get(&client_id)
+                        .cloned()
+                        .unwrap_or_else(|| self.default_keybinds.clone());
                     async move {
                         let _ = senders.send_to_background_jobs(
                             BackgroundJob::AnimatePluginLoading(plugin_id),
@@ -215,7 +240,8 @@ impl WasmBridge {
                                 .map(ToString::to_string)
                                 .collect();
 
-                            let downloader = Downloader::new(ZELLIJ_CACHE_DIR.to_path_buf());
+                            // if the url is already in cache, we'll use that version, otherwise
+                            // we'll download it, place it in cache and then use it
                             match downloader.download(url, Some(&file_name)).await {
                                 Ok(_) => plugin.path = ZELLIJ_CACHE_DIR.join(&file_name),
                                 Err(e) => handle_plugin_loading_failure(
@@ -236,8 +262,8 @@ impl WasmBridge {
                             plugin_dir,
                             plugin_cache,
                             senders.clone(),
-                            store,
-                            plugin_map,
+                            engine,
+                            plugin_map.clone(),
                             size,
                             connected_clients.clone(),
                             &mut loading_indication,
@@ -249,8 +275,13 @@ impl WasmBridge {
                             default_layout,
                             skip_cache,
                             layout_dir,
+                            default_mode,
+                            keybinds,
                         ) {
-                            Ok(_) => handle_plugin_successful_loading(&senders, plugin_id),
+                            Ok(_) => {
+                                let plugin_list = plugin_map.lock().unwrap().list_plugins();
+                                handle_plugin_successful_loading(&senders, plugin_id, plugin_list);
+                            },
                             Err(e) => handle_plugin_loading_failure(
                                 &senders,
                                 plugin_id,
@@ -291,7 +322,7 @@ impl WasmBridge {
                 drop(worker_sender.send(MessageToWorker::Exit));
             }
             let running_plugin = running_plugin.lock().unwrap();
-            let cache_dir = running_plugin.plugin_env.plugin_own_data_dir.clone();
+            let cache_dir = running_plugin.store.data().plugin_own_data_dir.clone();
             if let Err(e) = std::fs::remove_dir_all(cache_dir) {
                 log::error!("Failed to remove cache dir for plugin: {:?}", e);
             }
@@ -304,6 +335,80 @@ impl WasmBridge {
                 .send_to_server(ServerInstruction::UnblockCliPipeInput(pipe_name))
                 .context("failed to unblock input pipe");
         }
+        let plugin_list = plugin_map.list_plugins();
+        let _ = self
+            .senders
+            .send_to_background_jobs(BackgroundJob::ReportPluginList(plugin_list));
+        Ok(())
+    }
+    pub fn reload_plugin_with_id(&mut self, plugin_id: u32) -> Result<()> {
+        let Some(run_plugin) = self.run_plugin_of_plugin_id(plugin_id).map(|r| r.clone()) else {
+            log::error!("Failed to find plugin with id: {}", plugin_id);
+            return Ok(());
+        };
+
+        let (rows, columns) = self.size_of_plugin_id(plugin_id).unwrap_or((0, 0));
+        self.cached_events_for_pending_plugins
+            .insert(plugin_id, vec![]);
+        self.cached_resizes_for_pending_plugins
+            .insert(plugin_id, (rows, columns));
+
+        let mut loading_indication = LoadingIndication::new(run_plugin.location.to_string());
+        self.start_plugin_loading_indication(&[plugin_id], &loading_indication);
+        let load_plugin_task = task::spawn({
+            let plugin_dir = self.plugin_dir.clone();
+            let plugin_cache = self.plugin_cache.clone();
+            let senders = self.senders.clone();
+            let engine = self.engine.clone();
+            let plugin_map = self.plugin_map.clone();
+            let connected_clients = self.connected_clients.clone();
+            let path_to_default_shell = self.path_to_default_shell.clone();
+            let zellij_cwd = self.zellij_cwd.clone();
+            let capabilities = self.capabilities.clone();
+            let client_attributes = self.client_attributes.clone();
+            let default_shell = self.default_shell.clone();
+            let default_layout = self.default_layout.clone();
+            let layout_dir = self.layout_dir.clone();
+            async move {
+                match PluginLoader::reload_plugin(
+                    plugin_id,
+                    plugin_dir.clone(),
+                    plugin_cache.clone(),
+                    senders.clone(),
+                    engine.clone(),
+                    plugin_map.clone(),
+                    connected_clients.clone(),
+                    &mut loading_indication,
+                    path_to_default_shell.clone(),
+                    zellij_cwd.clone(),
+                    capabilities.clone(),
+                    client_attributes.clone(),
+                    default_shell.clone(),
+                    default_layout.clone(),
+                    layout_dir.clone(),
+                ) {
+                    Ok(_) => {
+                        let plugin_list = plugin_map.lock().unwrap().list_plugins();
+                        handle_plugin_successful_loading(&senders, plugin_id, plugin_list);
+                    },
+                    Err(e) => {
+                        handle_plugin_loading_failure(
+                            &senders,
+                            plugin_id,
+                            &mut loading_indication,
+                            &e,
+                            None,
+                        );
+                    },
+                }
+                let _ = senders.send_to_plugin(PluginInstruction::ApplyCachedEvents {
+                    plugin_ids: vec![plugin_id],
+                    done_receiving_permissions: false,
+                });
+            }
+        });
+        self.loading_plugins
+            .insert((plugin_id, run_plugin.clone()), load_plugin_task);
         Ok(())
     }
     pub fn reload_plugin(&mut self, run_plugin: &RunPlugin) -> Result<()> {
@@ -330,7 +435,7 @@ impl WasmBridge {
             let plugin_dir = self.plugin_dir.clone();
             let plugin_cache = self.plugin_cache.clone();
             let senders = self.senders.clone();
-            let store = self.store.clone();
+            let engine = self.engine.clone();
             let plugin_map = self.plugin_map.clone();
             let connected_clients = self.connected_clients.clone();
             let path_to_default_shell = self.path_to_default_shell.clone();
@@ -346,7 +451,7 @@ impl WasmBridge {
                     plugin_dir.clone(),
                     plugin_cache.clone(),
                     senders.clone(),
-                    store.clone(),
+                    engine.clone(),
                     plugin_map.clone(),
                     connected_clients.clone(),
                     &mut loading_indication,
@@ -359,7 +464,8 @@ impl WasmBridge {
                     layout_dir.clone(),
                 ) {
                     Ok(_) => {
-                        handle_plugin_successful_loading(&senders, first_plugin_id);
+                        let plugin_list = plugin_map.lock().unwrap().list_plugins();
+                        handle_plugin_successful_loading(&senders, first_plugin_id, plugin_list);
                         for plugin_id in &plugin_ids {
                             if plugin_id == &first_plugin_id {
                                 // no need to reload the plugin we just reloaded
@@ -371,7 +477,7 @@ impl WasmBridge {
                                 plugin_dir.clone(),
                                 plugin_cache.clone(),
                                 senders.clone(),
-                                store.clone(),
+                                engine.clone(),
                                 plugin_map.clone(),
                                 connected_clients.clone(),
                                 &mut loading_indication,
@@ -383,7 +489,14 @@ impl WasmBridge {
                                 default_layout.clone(),
                                 layout_dir.clone(),
                             ) {
-                                Ok(_) => handle_plugin_successful_loading(&senders, *plugin_id),
+                                Ok(_) => {
+                                    let plugin_list = plugin_map.lock().unwrap().list_plugins();
+                                    handle_plugin_successful_loading(
+                                        &senders,
+                                        *plugin_id,
+                                        plugin_list,
+                                    );
+                                },
                                 Err(e) => handle_plugin_loading_failure(
                                     &senders,
                                     *plugin_id,
@@ -423,7 +536,7 @@ impl WasmBridge {
             self.plugin_dir.clone(),
             self.plugin_cache.clone(),
             self.senders.clone(),
-            self.store.clone(),
+            self.engine.clone(),
             self.plugin_map.clone(),
             self.connected_clients.clone(),
             &mut loading_indication,
@@ -434,6 +547,11 @@ impl WasmBridge {
             self.default_shell.clone(),
             self.default_layout.clone(),
             self.layout_dir.clone(),
+            self.default_mode,
+            self.keybinds
+                .get(&client_id)
+                .cloned()
+                .unwrap_or_else(|| self.default_keybinds.clone()),
         ) {
             Ok(_) => {
                 let _ = self
@@ -487,27 +605,23 @@ impl WasmBridge {
                             running_plugin.rows = new_rows;
                             running_plugin.columns = new_columns;
 
-                            if old_rows != new_rows || old_columns != new_columns {
+                            // in the below conditional, we check if event_id == 0 so that we'll
+                            // make sure to always render on the first resize event
+                            if old_rows != new_rows || old_columns != new_columns || event_id == 0 {
                                 let rendered_bytes = running_plugin
                                     .instance
                                     .clone()
-                                    .exports
-                                    .get_function("render")
-                                    .map_err(anyError::new)
+                                    .get_typed_func::<(i32, i32), ()>(
+                                        &mut running_plugin.store,
+                                        "render",
+                                    )
                                     .and_then(|render| {
-                                        render
-                                            .call(
-                                                &mut running_plugin.store,
-                                                &[
-                                                    Value::I32(new_rows as i32),
-                                                    Value::I32(new_columns as i32),
-                                                ],
-                                            )
-                                            .map_err(anyError::new)
-                                    })
-                                    .and_then(|_| {
-                                        wasi_read_string(&running_plugin.plugin_env.wasi_env)
+                                        render.call(
+                                            &mut running_plugin.store,
+                                            (new_rows as i32, new_columns as i32),
+                                        )
                                     })
+                                    .and_then(|_| wasi_read_string(running_plugin.store.data()))
                                     .with_context(err_context);
                                 match rendered_bytes {
                                     Ok(rendered_bytes) => {
@@ -543,8 +657,6 @@ impl WasmBridge {
         mut updates: Vec<(Option<PluginId>, Option<ClientId>, Event)>,
         shutdown_sender: Sender<()>,
     ) -> Result<()> {
-        let err_context = || "failed to update plugin state".to_string();
-
         let plugins_to_update: Vec<(
             PluginId,
             ClientId,
@@ -563,57 +675,62 @@ impl WasmBridge {
                     .contains_key(&plugin_id)
             })
             .collect();
-        for (pid, cid, event) in updates.drain(..) {
-            for (plugin_id, client_id, running_plugin, subscriptions) in &plugins_to_update {
-                let subs = subscriptions.lock().unwrap().clone();
-                // FIXME: This is very janky... Maybe I should write my own macro for Event -> EventType?
-                let event_type =
-                    EventType::from_str(&event.to_string()).with_context(err_context)?;
-                if (subs.contains(&event_type) || event_type == EventType::PermissionRequestResult)
-                    && Self::message_is_directed_at_plugin(pid, cid, plugin_id, client_id)
-                {
-                    task::spawn({
-                        let senders = self.senders.clone();
-                        let running_plugin = running_plugin.clone();
-                        let event = event.clone();
-                        let plugin_id = *plugin_id;
-                        let client_id = *client_id;
-                        let _s = shutdown_sender.clone();
-                        async move {
-                            let mut running_plugin = running_plugin.lock().unwrap();
-                            let mut plugin_render_assets = vec![];
-                            let _s = _s; // guard to allow the task to complete before cleanup/shutdown
-                            match apply_event_to_plugin(
-                                plugin_id,
-                                client_id,
-                                &mut running_plugin,
-                                &event,
-                                &mut plugin_render_assets,
-                                senders.clone(),
-                            ) {
-                                Ok(()) => {
-                                    let _ = senders.send_to_screen(ScreenInstruction::PluginBytes(
-                                        plugin_render_assets,
-                                    ));
-                                },
-                                Err(e) => {
-                                    log::error!("{:?}", e);
+        task::spawn({
+            let mut updates = updates.clone();
+            let senders = self.senders.clone();
+            let s = shutdown_sender.clone();
+            async move {
+                let _s = s;
+                for (pid, cid, event) in updates.drain(..) {
+                    for (plugin_id, client_id, running_plugin, subscriptions) in &plugins_to_update
+                    {
+                        let subs = subscriptions.lock().unwrap().clone();
+                        // FIXME: This is very janky... Maybe I should write my own macro for Event -> EventType?
+                        if let Ok(event_type) = EventType::from_str(&event.to_string()) {
+                            if (subs.contains(&event_type)
+                                || event_type == EventType::PermissionRequestResult)
+                                && Self::message_is_directed_at_plugin(
+                                    pid, cid, plugin_id, client_id,
+                                )
+                            {
+                                let mut running_plugin = running_plugin.lock().unwrap();
+                                let mut plugin_render_assets = vec![];
+                                match apply_event_to_plugin(
+                                    *plugin_id,
+                                    *client_id,
+                                    &mut running_plugin,
+                                    &event,
+                                    &mut plugin_render_assets,
+                                    senders.clone(),
+                                ) {
+                                    Ok(()) => {
+                                        let _ = senders.send_to_screen(
+                                            ScreenInstruction::PluginBytes(plugin_render_assets),
+                                        );
+                                    },
+                                    Err(e) => {
+                                        log::error!("{:?}", e);
 
-                                    // https://stackoverflow.com/questions/66450942/in-rust-is-there-a-way-to-make-literal-newlines-in-r-using-windows-c
-                                    let stringified_error =
-                                        format!("{:?}", e).replace("\n", "\n\r");
+                                        // https://stackoverflow.com/questions/66450942/in-rust-is-there-a-way-to-make-literal-newlines-in-r-using-windows-c
+                                        let stringified_error =
+                                            format!("{:?}", e).replace("\n", "\n\r");
 
-                                    handle_plugin_crash(
-                                        plugin_id,
-                                        stringified_error,
-                                        senders.clone(),
-                                    );
-                                },
+                                        handle_plugin_crash(
+                                            *plugin_id,
+                                            stringified_error,
+                                            senders.clone(),
+                                        );
+                                    },
+                                }
                             }
                         }
-                    });
+                    }
                 }
             }
+        });
+        // loop once more to update the cached events for the pending plugins (probably currently
+        // being loaded, we'll send them these events when they load)
+        for (pid, _cid, event) in updates.drain(..) {
             for (plugin_id, cached_events) in self.cached_events_for_pending_plugins.iter_mut() {
                 if pid.is_none() || pid.as_ref() == Some(plugin_id) {
                     cached_events.push(EventOrPipeMessage::Event(event.clone()));
@@ -789,6 +906,55 @@ impl WasmBridge {
             .unwrap()
             .run_plugin_of_plugin_id(plugin_id)
     }
+
+    pub fn reconfigure(
+        &mut self,
+        client_id: ClientId,
+        keybinds: Option<Keybinds>,
+        default_mode: Option<InputMode>,
+        default_shell: Option<TerminalAction>,
+    ) -> Result<()> {
+        let plugins_to_reconfigure: Vec<Arc<Mutex<RunningPlugin>>> = self
+            .plugin_map
+            .lock()
+            .unwrap()
+            .running_plugins()
+            .iter()
+            .cloned()
+            .filter_map(|(_plugin_id, c_id, running_plugin)| {
+                if c_id == client_id {
+                    Some(running_plugin.clone())
+                } else {
+                    None
+                }
+            })
+            .collect();
+        if let Some(default_mode) = default_mode.as_ref() {
+            self.base_modes.insert(client_id, *default_mode);
+        }
+        if let Some(keybinds) = keybinds.as_ref() {
+            self.keybinds.insert(client_id, keybinds.clone());
+        }
+        self.default_shell = default_shell.clone();
+        for running_plugin in plugins_to_reconfigure {
+            task::spawn({
+                let running_plugin = running_plugin.clone();
+                let keybinds = keybinds.clone();
+                let default_shell = default_shell.clone();
+                async move {
+                    let mut running_plugin = running_plugin.lock().unwrap();
+                    if let Some(keybinds) = keybinds {
+                        running_plugin.update_keybinds(keybinds);
+                    }
+                    if let Some(default_mode) = default_mode {
+                        running_plugin.update_default_mode(default_mode);
+                    }
+                    running_plugin.update_default_shell(default_shell);
+                }
+            });
+        }
+        Ok(())
+    }
     fn apply_cached_events_and_resizes_for_plugin(
         &mut self,
         plugin_id: PluginId,
@@ -1075,12 +1241,13 @@ impl WasmBridge {
         };
 
         running_plugin
-            .plugin_env
+            .store
+            .data_mut()
             .set_permissions(HashSet::from_iter(permissions.clone()));
 
         let mut permission_cache = PermissionCache::from_path_or_default(cache_path);
         permission_cache.cache(
-            running_plugin.plugin_env.plugin.location.to_string(),
+            running_plugin.store.data().plugin.location.to_string(),
             permissions,
         );
 
@@ -1132,6 +1299,7 @@ impl WasmBridge {
                         cli_client_id,
                     ) {
                         Ok((plugin_id, client_id)) => {
+                            let start_suppressed = false;
                             drop(self.senders.send_to_screen(ScreenInstruction::AddPlugin(
                                 Some(should_float),
                                 should_be_open_in_place,
@@ -1141,6 +1309,7 @@ impl WasmBridge {
                                 plugin_id,
                                 pane_id_to_replace,
                                 cwd,
+                                start_suppressed,
                                 Some(client_id),
                             )));
                             vec![(plugin_id, Some(client_id))]
@@ -1207,11 +1376,27 @@ impl WasmBridge {
             || (message_cid.is_none() && message_pid == Some(*plugin_id))
             || (message_cid == Some(*client_id) && message_pid == Some(*plugin_id))
     }
+    pub fn client_is_connected(&self, client_id: &ClientId) -> bool {
+        self.connected_clients.lock().unwrap().contains(client_id)
+    }
+    pub fn get_first_client_id(&self) -> Option<ClientId> {
+        self.connected_clients
+            .lock()
+            .unwrap()
+            .iter()
+            .next()
+            .copied()
+    }
 }
 
-fn handle_plugin_successful_loading(senders: &ThreadSenders, plugin_id: PluginId) {
+fn handle_plugin_successful_loading(
+    senders: &ThreadSenders,
+    plugin_id: PluginId,
+    plugin_list: BTreeMap<PluginId, RunPlugin>,
+) {
     let _ = senders.send_to_background_jobs(BackgroundJob::StopPluginLoadingAnimation(plugin_id));
     let _ = senders.send_to_screen(ScreenInstruction::RequestStateUpdateForPlugins);
+    let _ = senders.send_to_background_jobs(BackgroundJob::ReportPluginList(plugin_list));
 }
 
 fn handle_plugin_loading_failure(
@@ -1253,6 +1438,13 @@ fn check_event_permission(
         | Event::SessionUpdate(..)
         | Event::CopyToClipboard(..)
         | Event::SystemClipboardFailure
+        | Event::CommandPaneOpened(..)
+        | Event::CommandPaneExited(..)
+        | Event::PaneClosed(..)
+        | Event::EditPaneOpened(..)
+        | Event::EditPaneExited(..)
+        | Event::FailedToWriteConfigToDisk(..)
+        | Event::CommandPaneReRun(..)
         | Event::InputReceived => PermissionType::ReadApplicationState,
         _ => return (PermissionStatus::Granted, None),
     };
@@ -1275,30 +1467,37 @@ pub fn apply_event_to_plugin(
     senders: ThreadSenders,
 ) -> Result<()> {
     let instance = &running_plugin.instance;
-    let plugin_env = &running_plugin.plugin_env;
     let rows = running_plugin.rows;
     let columns = running_plugin.columns;
 
     let err_context = || format!("Failed to apply event to plugin {plugin_id}");
-    match check_event_permission(plugin_env, event) {
+    match check_event_permission(running_plugin.store.data(), event) {
         (PermissionStatus::Granted, _) => {
+            let mut event = event.clone();
+            if let Event::ModeUpdate(mode_info) = &mut event {
+                // we do this because there can be some cases where this event arrives here with
+                // the wrong keybindings or default mode (for example: when triggered from the CLI,
+                // where we do not know the target client_id and thus don't know if their keybindings are the
+                // default or if they have changed at runtime), the keybindings in running_plugin
+                // should always be up-to-date. Ideally, we would have changed the keybindings in
+                // ModeInfo to an Option, but alas - this is already part of our contract and that
+                // would be a breaking change.
+                mode_info.keybinds = running_plugin.store.data().keybinds.to_keybinds_vec();
+                mode_info.base_mode = Some(running_plugin.store.data().default_mode);
+            }
             let protobuf_event: ProtobufEvent = event
                 .clone()
                 .try_into()
                 .map_err(|e| anyhow!("Failed to convert to protobuf: {:?}", e))?;
             let update = instance
-                .exports
-                .get_function("update")
+                .get_typed_func::<(), i32>(&mut running_plugin.store, "update")
                 .with_context(err_context)?;
-            wasi_write_object(&plugin_env.wasi_env, &protobuf_event.encode_to_vec())
+            wasi_write_object(running_plugin.store.data(), &protobuf_event.encode_to_vec())
                 .with_context(err_context)?;
-            let update_return = update
-                .call(&mut running_plugin.store, &[])
+            let should_render = update
+                .call(&mut running_plugin.store, ())
                 .with_context(err_context)?;
-            let mut should_render = match update_return.get(0) {
-                Some(Value::I32(n)) => *n == 1,
-                _ => false,
-            };
+            let mut should_render = should_render == 1;
             if let Event::PermissionRequestResult(..) = event {
                 // we always render in this case, otherwise the request permission screen stays on
                 // screen
@@ -1306,18 +1505,11 @@ pub fn apply_event_to_plugin(
             }
             if rows > 0 && columns > 0 && should_render {
                 let rendered_bytes = instance
-                    .exports
-                    .get_function("render")
-                    .map_err(anyError::new)
+                    .get_typed_func::<(i32, i32), ()>(&mut running_plugin.store, "render")
                     .and_then(|render| {
-                        render
-                            .call(
-                                &mut running_plugin.store,
-                                &[Value::I32(rows as i32), Value::I32(columns as i32)],
-                            )
-                            .map_err(anyError::new)
+                        render.call(&mut running_plugin.store, (rows as i32, columns as i32))
                     })
-                    .and_then(|_| wasi_read_string(&plugin_env.wasi_env))
+                    .and_then(|_| wasi_read_string(running_plugin.store.data()))
                     .with_context(err_context)?;
                 let pipes_to_block_or_unblock = pipes_to_block_or_unblock(running_plugin, None);
                 let plugin_render_asset = PluginRenderAsset::new(
