@@ -5,6 +5,7 @@ use std::ops::RangeBounds;
 pub struct Text {
     text: String,
     selected: bool,
+    opaque: bool,
     indices: Vec<Vec<usize>>,
 }
 
@@ -16,6 +17,7 @@ impl Text {
         Text {
             text: content.to_string(),
             selected: false,
+            opaque: false,
             indices: vec![],
         }
     }
@@ -23,6 +25,10 @@ impl Text {
         self.selected = true;
         self
     }
+    pub fn opaque(mut self) -> Self {
+        self.opaque = true;
+        self
+    }
     pub fn color_indices(mut self, index_level: usize, mut indices: Vec<usize>) -> Self {
         self.pad_indices(index_level);
         self.indices
@@ -75,11 +81,18 @@ impl Text {
                     .join(",")
             ));
         }
-        if self.selected {
-            format!("x{}{}", indices, text)
-        } else {
-            format!("{}{}", indices, text)
+
+        let mut prefix = "".to_owned();
+
+        if self.opaque {
+            prefix = format!("z{}", prefix);
         }
+
+        if self.selected {
+            prefix = format!("x{}", prefix);
+        }
+
+        format!("{}{}{}", prefix, indices, text)
     }
 }
 
