@@ -353,6 +353,26 @@ pub enum ScreenContext {
     RenameSession,
     DumpLayoutToPlugin,
     ListClientsMetadata,
+    Reconfigure,
+    RerunCommandPane,
+    ResizePaneWithId,
+    EditScrollbackForPaneWithId,
+    WriteToPaneId,
+    MovePaneWithPaneId,
+    MovePaneWithPaneIdInDirection,
+    ClearScreenForPaneId,
+    ScrollUpInPaneId,
+    ScrollDownInPaneId,
+    ScrollToTopInPaneId,
+    ScrollToBottomInPaneId,
+    PageScrollUpInPaneId,
+    PageScrollDownInPaneId,
+    TogglePaneIdFullscreen,
+    TogglePaneEmbedOrEjectForPaneId,
+    CloseTabWithIndex,
+    BreakPanesToNewTab,
+    BreakPanesToTabWithIndex,
+    ListClientsToPlugin,
 }
 
 /// Stack call representations corresponding to the different types of [`PtyInstruction`]s.
@@ -375,6 +395,8 @@ pub enum PtyContext {
     FillPluginCwd,
     DumpLayoutToPlugin,
     ListClientsMetadata,
+    Reconfigure,
+    ListClientsToPlugin,
     Exit,
 }
 
@@ -382,10 +404,12 @@ pub enum PtyContext {
 #[derive(Debug, Clone, Copy, PartialEq, Serialize, Deserialize)]
 pub enum PluginContext {
     Load,
+    LoadBackgroundPlugin,
     Update,
     Render,
     Unload,
     Reload,
+    ReloadPluginWithId,
     Resize,
     Exit,
     AddClient,
@@ -408,6 +432,9 @@ pub enum PluginContext {
     KeybindPipe,
     DumpLayoutToPlugin,
     ListClientsMetadata,
+    Reconfigure,
+    FailedToWriteConfigToDisk,
+    ListClientsToPlugin,
 }
 
 /// Stack call representations corresponding to the different types of [`ClientInstruction`]s.
@@ -431,6 +458,7 @@ pub enum ClientContext {
     UnblockCliPipeInput,
     CliPipeOutput,
     QueryTerminalSize,
+    WriteConfigToDisk,
 }
 
 /// Stack call representations corresponding to the different types of [`ServerInstruction`]s.
@@ -454,6 +482,12 @@ pub enum ServerContext {
     CliPipeOutput,
     AssociatePipeWithClient,
     DisconnectAllClientsExcept,
+    ChangeMode,
+    ChangeModeForAllClients,
+    Reconfigure,
+    ConfigWrittenToDisk,
+    FailedToWriteConfigToDisk,
+    RebindKeys,
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Serialize, Deserialize)]
@@ -475,6 +509,7 @@ pub enum BackgroundJobContext {
     ReportLayoutInfo,
     RunCommand,
     WebRequest,
+    ReportPluginList,
     Exit,
 }
 
