@@ -1,20 +1,19 @@
 //! Trigger a command
-use crate::data::Direction;
+use crate::data::{Direction, OriginatingPlugin};
 use serde::{Deserialize, Serialize};
 use std::path::PathBuf;
 
 #[derive(Debug, Clone)]
 pub enum TerminalAction {
-    OpenFile(PathBuf, Option<usize>, Option<PathBuf>), // path to file (should be absolute), optional line_number and an
-    // optional cwd
+    OpenFile(OpenFilePayload),
     RunCommand(RunCommand),
 }
 
 impl TerminalAction {
     pub fn change_cwd(&mut self, new_cwd: PathBuf) {
         match self {
-            TerminalAction::OpenFile(_, _, cwd) => {
-                *cwd = Some(new_cwd);
+            TerminalAction::OpenFile(open_file_payload) => {
+                open_file_payload.cwd = Some(new_cwd);
             },
             TerminalAction::RunCommand(run_command) => {
                 run_command.cwd = Some(new_cwd);
@@ -23,6 +22,29 @@ impl TerminalAction {
     }
 }
 
+#[derive(Clone, Debug, Serialize, Deserialize, PartialEq, Eq)]
+pub struct OpenFilePayload {
+    pub path: PathBuf,
+    pub line_number: Option<usize>,
+    pub cwd: Option<PathBuf>,
+    pub originating_plugin: Option<OriginatingPlugin>,
+}
+
+impl OpenFilePayload {
+    pub fn new(path: PathBuf, line_number: Option<usize>, cwd: Option<PathBuf>) -> Self {
+        OpenFilePayload {
+            path,
+            line_number,
+            cwd,
+            originating_plugin: None,
+        }
+    }
+    pub fn with_originating_plugin(mut self, originating_plugin: OriginatingPlugin) -> Self {
+        self.originating_plugin = Some(originating_plugin);
+        self
+    }
+}
+
 #[derive(Clone, Debug, Deserialize, Default, Serialize, PartialEq, Eq)]
 pub struct RunCommand {
     #[serde(alias = "cmd")]
@@ -35,6 +57,8 @@ pub struct RunCommand {
     pub hold_on_close: bool,
     #[serde(default)]
     pub hold_on_start: bool,
+    #[serde(default)]
+    pub originating_plugin: Option<OriginatingPlugin>,
 }
 
 impl std::fmt::Display for RunCommand {
@@ -68,6 +92,8 @@ pub struct RunCommandAction {
     pub hold_on_close: bool,
     #[serde(default)]
     pub hold_on_start: bool,
+    #[serde(default)]
+    pub originating_plugin: Option<OriginatingPlugin>,
 }
 
 impl From<RunCommandAction> for RunCommand {
@@ -78,6 +104,7 @@ impl From<RunCommandAction> for RunCommand {
             cwd: action.cwd,
             hold_on_close: action.hold_on_close,
             hold_on_start: action.hold_on_start,
+            originating_plugin: action.originating_plugin,
         }
     }
 }
@@ -91,6 +118,7 @@ impl From<RunCommand> for RunCommandAction {
             direction: None,
             hold_on_close: run_command.hold_on_close,
             hold_on_start: run_command.hold_on_start,
+            originating_plugin: run_command.originating_plugin,
         }
     }
 }
