@@ -8,6 +8,8 @@
 //  place.
 //  If plugins should be able to depend on the layout system
 //  then [`zellij-utils`] could be a proper place.
+#[cfg(not(target_family = "wasm"))]
+use crate::downloader::Downloader;
 use crate::{
     data::{Direction, LayoutInfo},
     home::{default_layout_dir, find_default_config_dir},
@@ -18,6 +20,8 @@ use crate::{
     pane_size::{Constraint, Dimension, PaneGeom},
     setup::{self},
 };
+#[cfg(not(target_family = "wasm"))]
+use async_std::task;
 
 use std::cmp::Ordering;
 use std::fmt::{Display, Formatter};
@@ -500,6 +504,12 @@ impl PluginUserConfiguration {
         configuration.remove("direction");
         configuration.remove("floating");
         configuration.remove("move_to_focused_tab");
+        configuration.remove("launch_new");
+        configuration.remove("payload");
+        configuration.remove("skip_cache");
+        configuration.remove("title");
+        configuration.remove("in_place");
+        configuration.remove("skip_plugin_cache");
 
         PluginUserConfiguration(configuration)
     }
@@ -731,6 +741,19 @@ pub struct FloatingPaneLayout {
 }
 
 impl FloatingPaneLayout {
+    pub fn new() -> Self {
+        FloatingPaneLayout {
+            name: None,
+            height: None,
+            width: None,
+            x: None,
+            y: None,
+            run: None,
+            focus: None,
+            already_running: false,
+            pane_initial_contents: None,
+        }
+    }
     pub fn add_cwd_to_layout(&mut self, cwd: &PathBuf) {
         match self.run.as_mut() {
             Some(run) => run.add_cwd(cwd),
@@ -915,7 +938,16 @@ impl TiledPaneLayout {
                 .len()
                 .saturating_sub(successfully_ignored)
             {
-                if let Some(position) = run_instructions.iter().position(|i| i == &None) {
+                if let Some(position) = run_instructions.iter().position(|i| {
+                    match i {
+                        // this is because a bare CWD instruction should be overidden by a terminal
+                        // in run_instructions_to_ignore (for cases where the cwd for example comes
+                        // from a global layout cwd and the pane is actually just a bare pane that
+                        // wants to be overidden)
+                        Some(Run::Cwd(_)) | None => true,
+                        _ => false,
+                    }
+                }) {
                     run_instructions.remove(position);
                 }
             }
@@ -1117,6 +1149,7 @@ impl Layout {
         available_layouts.push(LayoutInfo::BuiltIn("strider".to_owned()));
         available_layouts.push(LayoutInfo::BuiltIn("disable-status-bar".to_owned()));
         available_layouts.push(LayoutInfo::BuiltIn("compact".to_owned()));
+        available_layouts.push(LayoutInfo::BuiltIn("classic".to_owned()));
         available_layouts.sort_by(|a, b| {
             let a_name = a.name();
             let b_name = b.name();
@@ -1134,26 +1167,43 @@ impl Layout {
         layout_dir: &Option<PathBuf>,
         layout_info: LayoutInfo,
     ) -> Result<Layout, ConfigError> {
+        let mut should_start_layout_commands_suspended = false;
         let (path_to_raw_layout, raw_layout, raw_swap_layouts) = match layout_info {
             LayoutInfo::File(layout_name_without_extension) => {
                 let layout_dir = layout_dir.clone().or_else(|| default_layout_dir());
-                Self::stringified_from_dir(
-                    &PathBuf::from(layout_name_without_extension),
-                    layout_dir.as_ref(),
-                )?
+                let (path_to_layout, stringified_layout, swap_layouts) =
+                    Self::stringified_from_dir(
+                        &PathBuf::from(layout_name_without_extension),
+                        layout_dir.as_ref(),
+                    )?;
+                (Some(path_to_layout), stringified_layout, swap_layouts)
             },
             LayoutInfo::BuiltIn(layout_name) => {
-                Self::stringified_from_default_assets(&PathBuf::from(layout_name))?
+                let (path_to_layout, stringified_layout, swap_layouts) =
+                    Self::stringified_from_default_assets(&PathBuf::from(layout_name))?;
+                (Some(path_to_layout), stringified_layout, swap_layouts)
             },
+            LayoutInfo::Url(url) => {
+                should_start_layout_commands_suspended = true;
+                (Some(url.clone()), Self::stringified_from_url(&url)?, None)
+            },
+            LayoutInfo::Stringified(stringified_layout) => (None, stringified_layout, None),
         };
-        Layout::from_kdl(
+        let mut layout = Layout::from_kdl(
             &raw_layout,
             path_to_raw_layout,
             raw_swap_layouts
                 .as_ref()
                 .map(|(r, f)| (r.as_str(), f.as_str())),
             None,
-        )
+        );
+        if should_start_layout_commands_suspended {
+            layout
+                .iter_mut()
+                .next()
+                .map(|l| l.recursively_add_start_suspended_including_template(Some(true)));
+        }
+        layout
     }
     pub fn stringified_from_path_or_default(
         layout_path: Option<&PathBuf>,
@@ -1179,6 +1229,23 @@ impl Layout {
             ),
         }
     }
+    #[cfg(not(target_family = "wasm"))]
+    pub fn stringified_from_url(url: &str) -> Result<String, ConfigError> {
+        let raw_layout = task::block_on(async move {
+            let download = Downloader::download_without_cache(url).await;
+            match download {
+                Ok(stringified) => Ok(stringified),
+                Err(e) => Err(ConfigError::DownloadError(format!("{}", e))),
+            }
+        })?;
+        Ok(raw_layout)
+    }
+    #[cfg(target_family = "wasm")]
+    pub fn stringified_from_url(_url: &str) -> Result<String, ConfigError> {
+        // silently fail - this should not happen in plugins and legacy architecture is hard
+        let raw_layout = String::new();
+        Ok(raw_layout)
+    }
     pub fn from_path_or_default(
         layout_path: Option<&PathBuf>,
         layout_dir: Option<PathBuf>,
@@ -1188,7 +1255,7 @@ impl Layout {
             Layout::stringified_from_path_or_default(layout_path, layout_dir)?;
         let layout = Layout::from_kdl(
             &raw_layout,
-            path_to_raw_layout,
+            Some(path_to_raw_layout),
             raw_swap_layouts
                 .as_ref()
                 .map(|(r, f)| (r.as_str(), f.as_str())),
@@ -1197,6 +1264,34 @@ impl Layout {
         let config = Config::from_kdl(&raw_layout, Some(config))?; // this merges the two config, with
         Ok((layout, config))
     }
+    #[cfg(not(target_family = "wasm"))]
+    pub fn from_url(url: &str, config: Config) -> Result<(Layout, Config), ConfigError> {
+        let raw_layout = task::block_on(async move {
+            let download = Downloader::download_without_cache(url).await;
+            match download {
+                Ok(stringified) => Ok(stringified),
+                Err(e) => Err(ConfigError::DownloadError(format!("{}", e))),
+            }
+        })?;
+        let mut layout = Layout::from_kdl(&raw_layout, Some(url.into()), None, None)?;
+        layout.recursively_add_start_suspended_including_template(Some(true));
+        let config = Config::from_kdl(&raw_layout, Some(config))?; // this merges the two config, with
+        Ok((layout, config))
+    }
+    pub fn from_stringified_layout(
+        stringified_layout: &str,
+        config: Config,
+    ) -> Result<(Layout, Config), ConfigError> {
+        let layout = Layout::from_kdl(&stringified_layout, None, None, None)?;
+        let config = Config::from_kdl(&stringified_layout, Some(config))?; // this merges the two config, with
+        Ok((layout, config))
+    }
+    #[cfg(target_family = "wasm")]
+    pub fn from_url(_url: &str, _config: Config) -> Result<(Layout, Config), ConfigError> {
+        Err(ConfigError::DownloadError(format!(
+            "Unsupported platform, cannot download layout from the web"
+        )))
+    }
     pub fn from_path_or_default_without_config(
         layout_path: Option<&PathBuf>,
         layout_dir: Option<PathBuf>,
@@ -1205,7 +1300,7 @@ impl Layout {
             Layout::stringified_from_path_or_default(layout_path, layout_dir)?;
         let layout = Layout::from_kdl(
             &raw_layout,
-            path_to_raw_layout,
+            Some(path_to_raw_layout),
             raw_swap_layouts
                 .as_ref()
                 .map(|(r, f)| (r.as_str(), f.as_str())),
@@ -1222,7 +1317,7 @@ impl Layout {
             Layout::stringified_from_default_assets(layout_name)?;
         let layout = Layout::from_kdl(
             &raw_layout,
-            path_to_raw_layout,
+            Some(path_to_raw_layout),
             raw_swap_layouts
                 .as_ref()
                 .map(|(r, f)| (r.as_str(), f.as_str())),
@@ -1237,7 +1332,7 @@ impl Layout {
         swap_layouts: Option<(&str, &str)>, // Option<path_to_swap_layout, stringified_swap_layout>
         cwd: Option<PathBuf>,
     ) -> Result<Layout, ConfigError> {
-        Layout::from_kdl(raw, path_to_raw_layout, swap_layouts, cwd)
+        Layout::from_kdl(raw, Some(path_to_raw_layout), swap_layouts, cwd)
     }
     pub fn stringified_from_dir(
         layout: &PathBuf,
@@ -1321,6 +1416,14 @@ impl Layout {
                     Self::stringified_compact_swap_from_assets()?,
                 )),
             )),
+            Some("classic") => Ok((
+                "Classic layout".into(),
+                Self::stringified_classic_from_assets()?,
+                Some((
+                    "Classiclayout swap".into(),
+                    Self::stringified_classic_swap_from_assets()?,
+                )),
+            )),
             Some("welcome") => Ok((
                 "Welcome screen layout".into(),
                 Self::stringified_welcome_from_assets()?,
@@ -1357,6 +1460,14 @@ impl Layout {
         Ok(String::from_utf8(setup::COMPACT_BAR_SWAP_LAYOUT.to_vec())?)
     }
 
+    pub fn stringified_classic_from_assets() -> Result<String, ConfigError> {
+        Ok(String::from_utf8(setup::CLASSIC_LAYOUT.to_vec())?)
+    }
+
+    pub fn stringified_classic_swap_from_assets() -> Result<String, ConfigError> {
+        Ok(String::from_utf8(setup::CLASSIC_SWAP_LAYOUT.to_vec())?)
+    }
+
     pub fn stringified_welcome_from_assets() -> Result<String, ConfigError> {
         Ok(String::from_utf8(setup::WELCOME_LAYOUT.to_vec())?)
     }
@@ -1390,7 +1501,23 @@ impl Layout {
             }
         }
     }
-
+    pub fn recursively_add_start_suspended_including_template(
+        &mut self,
+        start_suspended: Option<bool>,
+    ) {
+        if let Some((tiled_panes_template, floating_panes_template)) = self.template.as_mut() {
+            tiled_panes_template.recursively_add_start_suspended(start_suspended);
+            for floating_pane in floating_panes_template.iter_mut() {
+                floating_pane.add_start_suspended(start_suspended);
+            }
+        }
+        for (_tab_name, tiled_panes, floating_panes) in self.tabs.iter_mut() {
+            tiled_panes.recursively_add_start_suspended(start_suspended);
+            for floating_pane in floating_panes.iter_mut() {
+                floating_pane.add_start_suspended(start_suspended);
+            }
+        }
+    }
     fn swap_layout_and_path(path: &Path) -> Option<(String, String)> {
         // Option<path, stringified_swap_layout>
         let mut swap_layout_path = PathBuf::from(path);
@@ -1443,6 +1570,22 @@ impl Layout {
             }
         }
     }
+    pub fn pane_count(&self) -> usize {
+        let mut pane_count = 0;
+        if let Some((tiled_pane_layout, floating_panes)) = self.template.as_ref() {
+            pane_count += tiled_pane_layout.pane_count();
+            for _ in floating_panes {
+                pane_count += 1;
+            }
+        }
+        for (_, tiled_pane_layout, floating_panes) in &self.tabs {
+            pane_count += tiled_pane_layout.pane_count();
+            for _ in floating_panes {
+                pane_count += 1;
+            }
+        }
+        pane_count
+    }
 }
 
 fn split_space(
