@@ -155,6 +155,12 @@ pub struct Options {
     /// If true, will disable writing session metadata to disk
     #[clap(long, value_parser)]
     pub disable_session_metadata: Option<bool>,
+
+    /// Whether to enable support for the Kitty keyboard protocol (must also be supported by the
+    /// host terminal), defaults to true if the terminal supports it
+    #[clap(long, value_parser)]
+    #[serde(default)]
+    pub support_kitty_keyboard_protocol: Option<bool>,
 }
 
 #[derive(ArgEnum, Deserialize, Serialize, Debug, Clone, Copy, PartialEq)]
@@ -230,6 +236,9 @@ impl Options {
         let disable_session_metadata = other
             .disable_session_metadata
             .or(self.disable_session_metadata);
+        let support_kitty_keyboard_protocol = other
+            .support_kitty_keyboard_protocol
+            .or(self.support_kitty_keyboard_protocol);
 
         Options {
             simplified_ui,
@@ -258,6 +267,7 @@ impl Options {
             styled_underlines,
             serialization_interval,
             disable_session_metadata,
+            support_kitty_keyboard_protocol,
         }
     }
 
@@ -313,6 +323,9 @@ impl Options {
         let disable_session_metadata = other
             .disable_session_metadata
             .or(self.disable_session_metadata);
+        let support_kitty_keyboard_protocol = other
+            .support_kitty_keyboard_protocol
+            .or(self.support_kitty_keyboard_protocol);
 
         Options {
             simplified_ui,
@@ -341,6 +354,7 @@ impl Options {
             styled_underlines,
             serialization_interval,
             disable_session_metadata,
+            support_kitty_keyboard_protocol,
         }
     }
 
@@ -405,6 +419,7 @@ impl From<CliOptions> for Options {
             scrollback_lines_to_serialize: opts.scrollback_lines_to_serialize,
             styled_underlines: opts.styled_underlines,
             serialization_interval: opts.serialization_interval,
+            support_kitty_keyboard_protocol: opts.support_kitty_keyboard_protocol,
             ..Default::default()
         }
     }
